--[[

CONSTANTS FOR LABWS2.0

]]

types = {
	SHIP = 0,
	POOP = 1,
	ENEMY = 2,
	UPGRADE = 3,
	SHIELD = 4,
}

constants = {

	--GAME STUFF
	SCREEN_WIDTH = 1024,
	SCREEN_HEIGHT = 768,
	DEBUG = true,
	MAX_HEALTH_VERT = { 15, 25, 15, 35, 115, 35, 115, 25 },

	--SHIP STUFF
	SHIP_HEIGHT = 40,
	SHIP_WIDTH = 20,
	SLOWDOWN = 0.5,
	ROT_SLOWDOWN = 0.65,
	MAX_ROT_VEL = 16,
	MAX_LIN_VEL = 600,
	MAX_RELOAD_TIME = .15,
	THRUST = 120,
	TURN_RATE = 21,
	SHIP_VERTICES = { 10, -20, 0, 20, -10, -20},

	--POOP STUFF
	INIT_POOP_VEL = 1700,
	POOP_FADEOUT_THRESHOLD = 0.3,
	POOP_ACTIVE_THRESHOLD = 0.25,

	--ENEMY STUFF
	ENEMY_SPAWN_RATE = 0.25,
	ENEMY_DIAMETERS = {25, 15, 12, 10, 8, 7 },

	--STAR STUFF
	STAR_DIAMETER = 1,
	MAX_STAR_SPEED = 0.001,

	--UPGRADE STUFF
	UPGRADE_SPAWN_RATE = 0.01,
	UPGRADE_CUTOFFS = {0.2,  -- Gold
					   				 0.7, -- HealthUp
	                   0.9, -- MaxHealth
	                   1.0  -- MaxHealthUp
	                  },
	UPGRADE_DIAMETER = 12,
	UPGRADE_SPEED = 50,
	UPGRADE_LIFETIME = 12,
	UPGRADE_EMISSION_RATE = 25,
	SHIELD_DIAMETER = 32.5,
}

colors = {
	WHITE = {255,255,255,255},
	BLACK = {0,0,0,255},
	NO_COLOR = {255,255,255,0},
	HEALTH_GREEN = {50, 200, 50, 255},
	HEALTH_RED = {200, 50, 50, 255},
	SHIP1 = {123,183,244,255},
	POOP_BROWN = {105, 69, 5, 255},
	HEALTH_UP = {25, 150, 50, 255},
	MAX_HEALTH = {222, 35, 35, 255},
	MAX_HEALTH_UP = {0, 0, 0, 255},
	GOLD = {222, 222, 42, 255},
	SHIELD = {200, }
}

--[[

UTILITY FUNCTIONS FOR LABWS2.0

]]

-- computes an angle given an x and y vector
function get_angle(x_coord, y_coord)
	return math.atan(y_coord/x_coord)
end

-- deep copies a list
function copy(list)
	new_list = {}
	for index,val in ipairs(list) do
		new_list[index] = val
	end
	return new_list
end

function polar_to_cartesian(r,theta)
	return r * math.cos(theta),
			   r * math.sin(theta)
end

-- wraps a pair of coordinates around the screen
function overside(x, y)
	x = mod(x, constants.SCREEN_WIDTH)
	y = mod(y, constants.SCREEN_HEIGHT)
	return x,y
end

-- returns (x) mod (modulus) assuming that x is not TOO big or TOO small
function mod(x, modulus)
	if x > modulus then x = x - modulus
	elseif x < 0 then x = x + modulus end
	return x
end

-- snaps x back to min or max if it goes too far
function bound(x, minbound, maxbound)
	if x < minbound then x = minbound
	elseif x > maxbound then x = maxbound end
	return x
end

-- returns the distance squared between two points
function dist_sq(x1, y1, x2, y2)
	return (x1-x2)^2 + (y1-y2)^2
end

-- returns the distance between two points
function dist(x1, y1, x2, y2)
	return ((x1-x2)^2 + (y1-y2)^2)^(0.5)
end

-- returns a table with every function in each of the two argument tables
function all_values(a,b)
	c={}
	for m,n in ipairs(a) do
		table.insert(c,n)
	end
	for p,q in ipairs(b) do
		table.insert(c,q)
	end
	return c
end
