--[[

A GENERIC ENEMY

]]

local class = require 'middleclass'
require 'constants'
require 'sprites'
Enemy = class('Enemy', ColliderSprite)

function Enemy:initialize(hc, level)
	self.alive = true
	self.level = level
	self.objtype = types.ENEMY
	self.speed = 100 * self.level
	-- spawns on left or right
	local x
	local y
	local x_vel
	local y_vel
	if (math.random() > 0.5) then
		-- spawns on left
		y = math.random(constants.SCREEN_HEIGHT)
		y_vel = 0
		if (math.random() > 0.5) then
			x = 0
			x_vel = 200
		-- spawns on right
		else
			x = constants.SCREEN_WIDTH
			x_vel = -200
		end
	-- spawns on top or bottom
	else
		x = math.random(constants.SCREEN_WIDTH)
		x_vel = 0
		-- spawns on top
		if (math.random() > 0.5) then
			y = 0
			y_vel = 200
		-- spawns on bottom
		else
			y = constants.SCREEN_HEIGHT
			y_vel = -200
		end
	end
	local diameter = constants.ENEMY_DIAMETERS[level]
	shape = hc:addCircle(x,y,diameter)
	local line_color = colors.SHIP1
	local border_color = colors.BLACK
	ColliderSprite.initialize(self, shape, x_vel, y_vel, 1.0, 0, 1.0, line_color, border_color, nil)
end

function Enemy:kill()
	self.alive = false
end

function Enemy:move(dt, ship)
	self:target_ship(dt, ship)
	self.x_vel = self.speed * math.cos(self.angle)
	self.y_vel = self.speed * math.sin(self.angle)
	ColliderSprite.move(self, dt)
end

-- adjusts the enemies angling to target the player
function Enemy:target_ship(dt, ship)
	ideal_angle = math.atan((ship.y - self.y)/(ship.x - self.x))

	if self.angle < ideal_angle then
		self:turn(dt * math.pi/(9*self.level))
	elseif self.angle > ideal_angle then
		self:turn(-dt * math.pi/(9*self.level))
	end

end

function Enemy:draw()
	ColliderSprite.draw(self)
	love.graphics.circle('fill',self.x + 10*math.cos(self.angle), self.y + 10*math.sin(self.angle), 5)
end

function Enemy:collide(obj)

	if obj.objtype == types.POOP then
		self:kill()
		return 'splat', 'spray_p', 15
	end
	if obj.objtype == types.SHIP then
		self:kill()
		return 'crash'
	end
	if obj.objtype == types.SHIELD then
		self:kill()
		return 'crash'
	end

end
