--[[

MAIN FILE FOR LABWS2.0

]]

local class = require 'middleclass'
require 'slam'
HC = require 'hardoncollider'
require 'constants'
require 'ship'
require 'poop'
require 'enemy'
require 'starscape'
require 'upgrades'

function on_collide(dt, shape_1, shape_2)

	collisions = collisions + 1

	sprite_1 = shape_1.parent
	sprite_2 = shape_2.parent

	collide_fx(sprite_1, sprite_2)
	collide_fx(sprite_2, sprite_1)

end

function collide_fx(sprite_1, sprite_2)
	local sound, particle, num_particles = sprite_1:collide(sprite_2)
	if sound ~= nil then
		local m = sfx[sound]:play()
		m:setPosition(sprite_1.x/constants.SCREEN_WIDTH, sprite_1.y/constants.SCREEN_HEIGHT, 0)
	end
	if particle ~= nil then
		particles[particle]:setPosition(sprite_1.x, sprite_1.y)
		particles[particle]:emit(num_particles)
	end
end

function love.load(arg)
	hc = HC(50, on_collide)
	--imports beautiful sound effects
	farts = {
					love.audio.newSource("assets/sounds/wave/fart1.wav", "static"),
					love.audio.newSource("assets/sounds/wave/fart2.wav", "static"),
					love.audio.newSource("assets/sounds/wave/fart3.wav", "static"),
					love.audio.newSource("assets/sounds/wave/fart4.wav", "static"),
					love.audio.newSource("assets/sounds/wave/fart5.wav", "static"),
					love.audio.newSource("assets/sounds/wave/fart6.wav", "static"),
			}

	sfx =   {
					crash = love.audio.newSource("assets/sounds/wave/crash.wav", "static"),
					splat = love.audio.newSource("assets/sounds/wave/splat.wav", "static"),
					healthup = love.audio.newSource("assets/sounds/wave/powerup1.wav", "static"),
					maxhealth = love.audio.newSource("assets/sounds/wave/powerup2.wav", "static"),
					maxhealthup = love.audio.newSource("assets/sounds/wave/powerup3.wav", "static"),
					powerup = love.audio.newSource("assets/sounds/wave/powerup4.wav", "static"),
					money = love.audio.newSource("assets/sounds/wave/chaching.wav", "static"),

			}
	music = {
					title = love.audio.newSource("assets/sounds/ogg/title.ogg", "stream"),
					theme = love.audio.newSource("assets/sounds/ogg/Space_Nausea.ogg", "stream"),
	}
	music.title:setLooping(true)
	music.theme:setLooping(true)
	music.theme:play()

	-- loads the particle systems

	particles = {
					spray_p = love.graphics.newParticleSystem(love.graphics.newImage('assets/images/fade_circle.png'), 100)
	}
	ship_img = love.graphics.newImage("assets/images/ship.png")
	particles.spray_p:setBufferSize( 10 )
 	particles.spray_p:setParticleLifetime( 1 )
	particles.spray_p:setColors( 105, 69, 5, 255, 82, 38, 16, 44 )
	particles.spray_p:setSizes( 0, 1.2, 1.4 )
	particles.spray_p:setSpeed( 80, 105)
	particles.spray_p:setSpread( 2*math.pi )

	-- initializes the important objects in the game
	player = Ship(hc, constants.SCREEN_WIDTH/2, constants.SCREEN_HEIGHT/2, 0, 0, 0, ship_img)
	-- p.s. all objects that implement the collider are going to have to have it as an argument
	-- (i've changed this throughout the rest of the code as well)
	poops = {}
	enemies = {}
	starscape = Starscape(200)
	upgrades = {}

	-- loads the stats for the game
	score = 0
	collisions = 0


end


--[[ CONTROLS FOR THE SHIP ]]
function love.update(dt)
	delTime = dt
	hc:update(dt)

	-- left/right turns the ship
	if love.keyboard.isDown('left') then
		player:turn(-1*constants.TURN_RATE*dt)
	end
	if love.keyboard.isDown('right') then
		player:turn(constants.TURN_RATE*dt)
	end

	-- up makes the ship poop
	if love.keyboard.isDown('up') then
		p = player:poop(hc)
		if p ~= nil then
			f = farts[math.random(#farts)]:play()
			f:setPosition(player.x/constants.SCREEN_WIDTH,player.y/constants.SCREEN_HEIGHT,0)
			table.insert(poops, p)
		end
	end


	--[[ SPAWNING FOR THE ENEMIES AND UPGRADES ]]
	--check to see whether or not to spawn an enemy
	if (math.random() < (constants.ENEMY_SPAWN_RATE*dt)) then
		table.insert(enemies, Enemy(hc, 1))
	end
		--check to see whether or not to spawn an upgrade
	--if love.keyboard.isDown('down') then
	if (math.random() < (constants.UPGRADE_SPAWN_RATE*dt)) then
		table.insert(upgrades, get_upgrade())
	end



	--[[ EVERYONE MOVES AND DIES ]]
	-- update the position of the ship and the poops
	player:move(dt)
	starscape:move(dt, player.x_vel, player.y_vel)

	for i = #poops, 1, -1 do
		poop = poops[i]
		if poop.alive then
			poop:move(dt)
		else
			hc:remove(poop.shape)
			table.remove(poops, i)
		end
	end
	for i = #enemies, 1, -1 do
		enemy = enemies[i]
		if enemy.alive then
			enemy:move(dt,player)
		else
			hc:remove(enemy.shape)
			table.remove(enemies, i)
		end
	end
	for i = #upgrades, 1, -1 do
		upgrade = upgrades[i]
		if upgrade.alive then
			upgrade:move(dt)
		else
			hc:remove(upgrade.shape)
			table.remove(upgrades, i)
		end
	end

	--update all particle effects
	for a, b in pairs(particles) do
		b:update(dt)
	end
end

function love.draw()
	love.graphics.setColor(colors.WHITE)
	if constants.DEBUG then
		-- print how many poops there are

		love.graphics.print('#poops', 15,40)
		love.graphics.print(#poops, 115,40)
		-- print how many enemies there are
		love.graphics.print('#enemies', 15,50)
		love.graphics.print(#enemies, 115, 50)
		-- print how many stars there are
		love.graphics.print('#stars', 15,60)
		love.graphics.print(#(starscape.stars), 115, 60)
		-- print how many upgrades there are
		love.graphics.print('#upgrades', 15,70)
		love.graphics.print(#upgrades, 115, 70)
		-- print dt
		love.graphics.print('dt', 15, 80)
		love.graphics.print(delTime, 115, 80)
		-- print how many upgrades there are
		love.graphics.print('#collisions', 15,90)
		love.graphics.print(collisions, 115, 90)

		-- print all the ship's information
		love.graphics.print(player.x, 870, 0)
		love.graphics.print(player.y, 870, 10)

		love.graphics.print(player.x_vel, 870, 20)
		love.graphics.print(player.y_vel, 870, 30)
		love.graphics.print(player.lin_slow, 870, 40)
		love.graphics.print(player.shape:rotation(), 870, 50)
		love.graphics.print(player.rot_vel, 870, 60)
		love.graphics.print(player.rot_slow, 870, 70)
		--for ind,vert in ipairs(player.vertices) do
		--	love.graphics.print(vert, 870, 70+10*ind)
		--end
		--for ind, vert in player.shape:unpack() do
		--	love.graphics.print(vert, 870, 70 + 10 * ind)
		--`send
		love.graphics.print('player.x', 720, 0)
		love.graphics.print('player.y', 720, 10)
		love.graphics.print('player.x_vel', 720, 20)
		love.graphics.print('player.y_vel', 720, 30)
		love.graphics.print('player.lin_slow', 720, 40)
		love.graphics.print('player.angle', 720, 50)
		love.graphics.print('player.rot_vel', 720, 60)
		love.graphics.print('player.rot_slow', 720, 70)
	end

	-- FIRST draw the stars to the screen
	starscape:draw()

	-- then draw the poopspray
	for a, b in pairs(particles) do
		love.graphics.draw(b,0,0)
	end

	-- then draw the score and the health bar
	love.graphics.print('SCORE: ', 35, 10)
	love.graphics.print(score, 85, 10)
	if (player.health > (0.2*player.max_health)) then
		love.graphics.setColor(colors.HEALTH_GREEN)
	else
		love.graphics.setColor(colors.HEALTH_RED)
	end
	love.graphics.polygon("fill",get_health_vert(player.health))
	love.graphics.setColor(colors.WHITE)
	love.graphics.polygon("line",get_health_vert(player.max_health))


	-- draw all of the poops to the screen
	for ind,poop in ipairs(poops) do
		poop:draw()
	end

	-- draw all of the upgrades to the screen
	for ind,upgrade in ipairs(upgrades) do
		upgrade:draw()
	end

	-- draw all of the enemies to the screen
	for ind,enemy in ipairs(enemies) do
		enemy:draw()
	end

	-- draw the ship to the screen
	player:draw()

end

--[[ FUNCTIONS USEFUL AT THIS LEVEL OF THE GAME ]]

function get_health_vert(health)
	return { 15, 25, 15, 35, (15+10*health), 35, (15+10*health), 25 }
end
