--[[

THE POOP

]]

local class = require 'middleclass'
require 'constants'
require 'sprites'
Poop = class('Poop', ColliderSprite)

function Poop:initialize(x, y, diameter, x_vel, y_vel,lifetime, hc)
	self.objtype = types.POOP
	self.diameter = diameter
	local shape = hc:addCircle(x,y,diameter)

	ColliderSprite.initialize(self, shape, x_vel, y_vel, 1.0, 0, 1.0, colors.POOP_BROWN, colors.NO_COLOR, nil)
	self.lifetime = lifetime
	self.alive = true
	--deep copies poop color because lua has no static variables
	self.active_counter = 0
	self.active = false
end

function Poop:move(dt)
	ColliderSprite.move(self, dt)
	self.lifetime = self.lifetime - dt
	if self.lifetime < constants.POOP_FADEOUT_THRESHOLD then
		--decrease the alpha of the poop until it is invisible
		self.color[4] = self.color[4] * 0.95
	end
	if self.lifetime < 0 then
		--delete the poop from the poops array once it times out
		self:kill()
	end
	if not self.active then
		if self.active_counter < constants.POOP_ACTIVE_THRESHOLD then
			self.active_counter = self.active_counter + dt
		else self.active = true
		end
	end
end


function Poop:kill()
	self.alive = false
end


function Poop:collide(obj)

	if obj.objtype == types.SHIP then
		if self.active then
			self:kill()
			return 'crash'
		end
	end
	if obj.objtype == types.ENEMY then
		self:kill()
	end
	if obj.objtype == types.SHIELD then
		self:kill()
		return 'bounce'
	end
end
