--[[

THE SHIP

]]

local class = require 'middleclass'
require 'constants'
require 'poop'
require 'sprites'
require 'shield'
Ship = class('Ship', ColliderSprite)

-- the ship starts in the middle of the screen without moving
function Ship:initialize(hc, x, y, x_vel, y_vel, rot_vel, ship_img)
	self.objtype = types.SHIP
	local vertices = constants.SHIP_VERTICES
	local shape = hc:addPolygon(vertices[1] + x, vertices[2] + y,
								vertices[3] + x, vertices[4] + y,
								vertices[5] + x, vertices[6] + y)
	ColliderSprite.initialize(self, shape, x_vel, y_vel, constants.SLOWDOWN, 0, constants.ROT_SLOWDOWN, colors.WHITE, colors.SHIP1,nil)-- ship_img)
	self.reload_timer = 0
	self.health = 10
	self.max_health = 10
	self.shield = Shield()
end

-- updates the ships position and velocities
function Ship:move(dt)
	ColliderSprite.move(self, dt)
	-- resets the timer so that the ship can eventually poop again
	if self.reload_timer >= 0 then self.reload_timer = self.reload_timer - dt end
	if self.shield.on then self.shield:update(dt, self.x, self.y) end
end

-- angularly accelerates the ship
function Ship:turn(deltaRotVel)
	ColliderSprite.turn(self, deltaRotVel)
	self.rot_vel = bound(self.rot_vel, -constants.MAX_ROT_VEL, constants.MAX_ROT_VEL)
end

-- propels the ship by launching a poop
function Ship:poop(hc)

	-- if it has been long enough since our last poop
	if self.reload_timer < 0 then
		self.reload_timer = constants.MAX_RELOAD_TIME

		-- resets velocity according to the direction the poop was launched
		xnext = math.sin(self.angle)
		ynext = math.cos(self.angle)
		self.x_vel = self.x_vel - xnext * constants.THRUST
		self.y_vel = self.y_vel + ynext * constants.THRUST

		-- bounds the velocity
		self.x_vel = bound(self.x_vel, -constants.MAX_LIN_VEL, constants.MAX_LIN_VEL)
		self.y_vel = bound(self.y_vel, -constants.MAX_LIN_VEL, constants.MAX_LIN_VEL)

		-- determine the size of the poop and return it
		diameter = math.random(7,12)

		return Poop(self.x + constants.SHIP_HEIGHT/3 * math.sin(self.angle),
			        self.y - constants.SHIP_HEIGHT/3 * math.cos(self.angle),
			        diameter,
			        constants.INIT_POOP_VEL*xnext/diameter,
			        -constants.INIT_POOP_VEL*ynext/diameter,
			        diameter/2.5, hc)
	end

	-- else we have to wait before we can poop again
	return nil
end

-- damages the ship after it gets hit by something
function Ship:damage(dm)
	self.health = self.health - dm
end

function Ship:alive()
	return self.health > 0
end

function Ship:draw()

	if self.shield.on then
		self.shield:draw()
	end
	ColliderSprite.draw(self)
end

function Ship:collide(obj)

	if obj.objtype == types.POOP then
		if obj.active then
			self:damage(1)
		end
	end
	if obj.objtype == types.ENEMY then
		self:damage(1)
	end

end
