--[[

A STAR

]]

local class = require 'middleclass'
require 'constants'
require 'sprites'
Star = class('Star', Sprite)

function Star:initialize(x, y, speed_mult)
	Sprite.initialize(self, x, y, 0, 0, 1.0)
	self.color = copy(colors.WHITE)
	self.speed_mult = speed_mult
	self.diameter = math.ceil(constants.STAR_DIAMETER*self.speed_mult*1200)

	for a = 1,2,3 do
		if (math.random() < 0.1) then
			self.color[a] = bound(self.color[a] - math.random(30 * (5-self.diameter)),
														200, 255)
		end
	end
end

function Star:move(dt, x_vel, y_vel)
	local total_vel = dist(x_vel, y_vel, 0, 0)
	self.x = self.x + (x_vel*dt)*self.speed_mult*(total_vel)^(0.65)
	self.y = self.y + (y_vel*dt)*self.speed_mult*(total_vel)^(0.65)

	self.x, self.y = overside(self.x, self.y)

end

function Star:draw()
	love.graphics.setColor(self.color)
	love.graphics.circle("fill", self.x, self.y, self.diameter)
end
